@@ -8,13 +8,13 @@
 from __future__ import absolute_import, division, print_function
 
 import math
-import numpy
+import numpy as np
 
 from ginga.AstroImage import AstroImage
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
                                        Point, MovePoint, ScalePoint,
                                        register_canvas_types, get_canvas_type,
-                                       colors_plus_none)
+                                       colors_plus_none, coord_names)
 from ginga.misc.ParamSet import Param
 from ginga.util import wcs
 from ginga.util.wcs import raDegToString, decDegToString
@@ -38,9 +38,9 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data', 'wcs'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x1', type=float, default=0.0, argpos=0,
                   description="First X coordinate of object"),
             Param(name='y1', type=float, default=0.0, argpos=1,
@@ -89,12 +89,12 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
                  showcap=True, showplumb=True, units='arcmin',
                  font='Sans Serif', fontsize=None, **kwdargs):
         self.kind = 'ruler'
+        points = np.asarray([(x1, y1), (x2, y2)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, color2=color2,
                                   alpha=alpha, units=units,
                                   showplumb=showplumb,
                                   linewidth=linewidth, showcap=showcap,
-                                  linestyle=linestyle,
-                                  x1=x1, y1=y1, x2=x2, y2=y2,
+                                  linestyle=linestyle, points=points,
                                   font=font, fontsize=fontsize,
                                   **kwdargs)
         TwoPointMixin.__init__(self)
@@ -104,11 +104,9 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
         points = self.get_data_points(points=points)
         return points
 
-    def select_contains(self, viewer, data_x, data_y):
+    def select_contains_pt(self, viewer, pt):
         points = self.get_points()
-        x1, y1 = points[0]
-        x2, y2 = points[1]
-        return self.within_line(viewer, data_x, data_y, x1, y1, x2, y2,
+        return self.within_line(viewer, pt, points[0], points[1],
                                 self.cap_radius)
 
     def get_ruler_distances(self, viewer):
@@ -150,7 +148,7 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
             else:
                 dx = abs(x2 - x1)
                 dy = abs(y2 - y1)
-                dh = math.sqrt(dx**2 + dy**2)
+                dh = np.sqrt(dx**2 + dy**2)
                 text_x = ("%.3f" % dx)
                 text_y = ("%.3f" % dy)
                 text_h = ("%.3f" % dh)
@@ -163,11 +161,15 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
         return (text_x, text_y, text_h)
 
     def draw(self, viewer):
+        image = viewer.get_image()
+        if image is None:
+            return
+
         points = self.get_points()
         x1, y1 = points[0]
         x2, y2 = points[1]
-        cx1, cy1 = self.canvascoords(viewer, x1, y1)
-        cx2, cy2 = self.canvascoords(viewer, x2, y2)
+        cx1, cy1 = viewer.get_canvas_xy(x1, y1)
+        cx2, cy2 = viewer.get_canvas_xy(x2, y2)
 
         text_x, text_y, text_h = self.get_ruler_distances(viewer)
 
@@ -253,9 +255,9 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -294,10 +296,11 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
                  linewidth=1, fontsize=None, font='Sans Serif',
                  alpha=1.0, linestyle='solid', showcap=True, **kwdargs):
         self.kind = 'compass'
+        points = np.asarray([(x, y)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, showcap=showcap,
                                   linestyle=linestyle,
-                                  x=x, y=y, radius=radius,
+                                  points=points, radius=radius,
                                   font=font, fontsize=fontsize,
                                   **kwdargs)
         OnePointOneRadiusMixin.__init__(self)
@@ -318,20 +321,22 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
 
     def set_edit_point(self, i, pt, detail):
         if i == 0:
-            x, y = pt
-            self.move_to(x, y)
+            self.move_to_pt(pt)
         elif i in (1, 2):
             x, y = pt
             self.radius = max(abs(x - self.x), abs(y - self.y))
         else:
             raise ValueError("No point corresponding to index %d" % (i))
 
-    def select_contains(self, viewer, data_x, data_y):
-        xd, yd = self.crdmap.to_data(self.x, self.y)
-        return self.within_radius(viewer, data_x, data_y, xd, yd,
-                                  self.cap_radius)
+    def select_contains_pt(self, viewer, pt):
+        p0 = self.crdmap.to_data((self.x, self.y))
+        return self.within_radius(viewer, pt, p0, self.cap_radius)
 
     def draw(self, viewer):
+        image = viewer.get_image()
+        if image is None:
+            return
+
         cr = viewer.renderer.setup_cr(self)
         cr.set_font_from_shape(self)
 
@@ -406,9 +411,9 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -450,17 +455,17 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
                  fontsize=None, font='Sans Serif', format='xy',
                  **kwdargs):
         self.kind = 'crosshair'
+        points = np.asarray([(x, y)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, linestyle=linestyle,
                                   text=text, textcolor=textcolor,
                                   fontsize=fontsize, font=font,
-                                  x=x, y=y, format=format, **kwdargs)
+                                  points=points, format=format, **kwdargs)
         OnePointMixin.__init__(self)
 
-    def select_contains(self, viewer, data_x, data_y):
-        xd, yd = self.crdmap.to_data(self.x, self.y)
-        return self.within_radius(viewer, data_x, data_y, xd, yd,
-                                  self.cap_radius)
+    def select_contains_pt(self, viewer, pt):
+        p0 = self.crdmap.to_data((self.x, self.y))
+        return self.within_radius(viewer, pt, p0, self.cap_radius)
 
     def draw(self, viewer):
         wd, ht = viewer.get_window_size()
@@ -478,6 +483,8 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
 
             else:
                 image = viewer.get_image()
+                if image is None:
+                    return
                 # NOTE: x, y are assumed to be in data coordinates
                 info = image.info_xy(self.x, self.y, viewer.get_settings())
                 if self.format == 'coords':
@@ -504,26 +511,26 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
 
 class AnnulusMixin(object):
 
-    def contains(self, x, y):
+    def contains_pt(self, pt):
         """Containment test."""
         obj1, obj2 = self.objects
-        return obj2.contains(x, y) and numpy.logical_not(obj1.contains(x, y))
+        return obj2.contains_pt(pt) and np.logical_not(obj1.contains_pt(pt))
 
-    def contains_arr(self, x_arr, y_arr):
+    def contains_pts(self, pts):
         """Containment test on arrays."""
         obj1, obj2 = self.objects
-        arg1 = obj2.contains_arr(x_arr, y_arr)
-        arg2 = numpy.logical_not(obj1.contains_arr(x_arr, y_arr))
-        return numpy.logical_and(arg1, arg2)
+        arg1 = obj2.contains_pts(pts)
+        arg2 = np.logical_not(obj1.contains_pts(pts))
+        return np.logical_and(arg1, arg2)
 
     def get_llur(self):
         """Bounded by outer object."""
         obj2 = self.objects[1]
         return obj2.get_llur()
 
-    def select_contains(self, viewer, data_x, data_y):
+    def select_contains_pt(self, viewer, pt):
         obj2 = self.objects[1]
-        return obj2.select_contains(viewer, data_x, data_y)
+        return obj2.select_contains_pt(viewer, pt)
 
 
 class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
@@ -541,9 +548,9 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data', 'wcs'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -573,8 +580,8 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
 
     @classmethod
     def idraw(cls, canvas, cxt):
-        radius = math.sqrt(abs(cxt.start_x - cxt.x)**2 +
-                           abs(cxt.start_y - cxt.y)**2)
+        radius = np.sqrt(abs(cxt.start_x - cxt.x)**2 +
+                         abs(cxt.start_y - cxt.y)**2)
         return cls(cxt.start_x, cxt.start_y, radius,
                    **cxt.drawparams)
 
@@ -606,8 +613,10 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
                      coord=coord)
         obj2.editable = False
 
+        points = np.asarray([(x, y)], dtype=np.float)
+
         CompoundObject.__init__(self, obj1, obj2,
-                                x=x, y=y, radius=radius,
+                                points=points, radius=radius,
                                 width=width, color=color,
                                 linewidth=linewidth, linestyle=linestyle,
                                 alpha=alpha, **kwdargs)
@@ -620,9 +629,9 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     def get_edit_points(self, viewer):
         points = ((self.x, self.y),
                   self.crdmap.offset_pt((self.x, self.y),
-                                        self.radius, 0),
+                                        (self.radius, 0)),
                   self.crdmap.offset_pt((self.x, self.y),
-                                        self.radius + self.width, 0),
+                                        (self.radius + self.width, 0)),
                   )
         points = self.get_data_points(points=points)
         return [MovePoint(*points[0]),
@@ -637,8 +646,7 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     def set_edit_point(self, i, pt, detail):
         if i == 0:
             # move control point
-            x, y = pt
-            self.move_to(x, y)
+            self.move_to_pt(pt)
         else:
             if i == 1:
                 scalef = self.calc_scale_from_pt(pt, detail)
@@ -662,7 +670,7 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         if oradius < self.radius:
             raise ValueError('Outer boundary < inner boundary')
 
-        d = dict(x=self.x, y=self.y, radius=self.radius, color=self.color,
+        d = dict(points=self.points, radius=self.radius, color=self.color,
                  linewidth=self.linewidth, linestyle=self.linestyle,
                  alpha=self.alpha)
 
@@ -673,10 +681,10 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         d['radius'] = oradius
         self.objects[1].__dict__.update(d)
 
-    def move_to(self, xdst, ydst):
-        super(Annulus, self).move_to(xdst, ydst)
+    def move_to_pt(self, dst_pt):
+        super(Annulus, self).move_to_pt(dst_pt)
 
-        self.set_data_points([(xdst, ydst)])
+        self.set_data_points([dst_pt])
 
 
 class WCSAxes(CompoundObject):
@@ -745,7 +753,7 @@ class WCSAxes(CompoundObject):
         xmax = image.width - 1
         ymax = image.height - 1
         try:
-            radec = image.wcs.datapt_to_coords(
+            radec = image.wcs.datapt_to_system(
                 [[0, 0], [0, ymax], [xmax, 0], [xmax, ymax]],
                 naxispath=image.naxispath)
         except Exception:
@@ -758,8 +766,8 @@ class WCSAxes(CompoundObject):
         # Calculate positions of RA/DEC lines
         d_ra = ra_size / (self.num_ra + 1)
         d_dec = dec_size / (self.num_dec + 1)
-        ra_arr = numpy.arange(ra_min + d_ra, ra_max - d_ra * 0.5, d_ra)
-        dec_arr = numpy.arange(dec_min + d_dec, dec_max - d_ra * 0.5, d_dec)
+        ra_arr = np.arange(ra_min + d_ra, ra_max - d_ra * 0.5, d_ra)
+        dec_arr = np.arange(dec_min + d_dec, dec_max - d_ra * 0.5, d_dec)
 
         # RA/DEC step size for each vector
         min_imsize = min(image.width, image.height)
@@ -771,12 +779,12 @@ class WCSAxes(CompoundObject):
 
         for cur_ra in ra_arr:
             crds = [[cur_ra, cur_dec] for cur_dec in
-                    numpy.arange(dec_min, dec_max + d_dec_step, d_dec_step)]
+                    np.arange(dec_min, dec_max + d_dec_step, d_dec_step)]
             lbl = raDegToString(cur_ra)
             objs += self._get_path(viewer, image, crds, lbl, 1)
         for cur_dec in dec_arr:
             crds = [[cur_ra, cur_dec] for cur_ra in
-                    numpy.arange(ra_min, ra_max + d_ra_step, d_ra_step)]
+                    np.arange(ra_min, ra_max + d_ra_step, d_ra_step)]
             lbl = decDegToString(cur_dec)
             objs += self._get_path(viewer, image, crds, lbl, 0)
 
@@ -816,7 +824,7 @@ class WCSAxes(CompoundObject):
                 y = m * x + c + self.txt_off
             else:  # y axis varying
                 y = min(y1, y2) + abs(dy) * 0.45
-                if numpy.isfinite(m):
+                if np.isfinite(m):
                     x = (y - c) / m
                 else:
                     x = min(x1, x2)
