@@ -339,7 +339,9 @@ class Desktop(Callback.Callbacks):
                         wexp = 0
                     if hexp is None:
                         hexp = 0
-                    widget.cfg_expand(wexp, hexp)
+                    # This is causing issues with resizing (see issue #478);
+                    # temporarily disabling it
+                    #widget.cfg_expand(wexp, hexp)
 
                 # User wants to place window somewhere
                 if params.xpos >= 0:
@@ -634,7 +636,7 @@ class Desktop(Callback.Callbacks):
                 layout = self.read_layout_conf(lo_file)
 
             except Exception as e:
-                self.logger.error("Error reading saved layout: %s" % (str(e)))
+                self.logger.info("Error reading saved layout: %s" % (str(e)))
                 layout = alt_layout
 
         return self.make_desktop(layout, widget_dict=widget_dict)
